@@ -24,13 +24,22 @@ import warnings
 
 from ..overrides import override, strip_boolean_result
 from ..module import get_introspection_module
-from gi import PyGIDeprecationWarning
+from gi import PyGIDeprecationWarning, require_version
 
 Gdk = get_introspection_module('Gdk')
 
 __all__ = []
 
 
+# https://bugzilla.gnome.org/show_bug.cgi?id=673396
+try:
+    require_version("GdkX11", Gdk._version)
+    from gi.repository import GdkX11
+    GdkX11  # pyflakes
+except (ValueError, ImportError):
+    pass
+
+
 class Color(Gdk.Color):
     MAX_VALUE = 65535
 
@@ -134,6 +143,14 @@ else:
         Rectangle = _cairo.RectangleInt
 
         __all__.append('Rectangle')
+    else:
+        # https://bugzilla.gnome.org/show_bug.cgi?id=756364
+        # These methods used to be functions, keep aliases for backwards compat
+        rectangle_intersect = Gdk.Rectangle.intersect
+        rectangle_union = Gdk.Rectangle.union
+
+        __all__.append('rectangle_intersect')
+        __all__.append('rectangle_union')
 
 if Gdk._version == '2.0':
     class Drawable(Gdk.Drawable):
